package de.meinefirma.meinprojekt;

import java.io.*;
import java.nio.charset.Charset;
import java.util.*;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.*;
import org.apache.lucene.index.*;
import org.apache.lucene.index.IndexWriterConfig.OpenMode;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.*;
import org.apache.lucene.store.*;
import org.apache.lucene.util.Version;

public class LuceneSimpleUtil
{
   public static void main( String[] args ) throws Exception
   {
      if( args.length != 2 && args.length != 3 ) {
         System.out.println(
               "\nIndizieren mit 3 Parametern:\n" + 
               "  PfadZuDokumenten CharacterEncoding PfadZumIndex\n" +
               "Suchen mit 2 Parametern:\n" + 
               "  PfadZumIndex SuchWort\n" );
      }
      if( args.length == 3 ) {
         System.out.println( "\nIndexerstellung im Verzeichnis:         " + args[2] );
         System.out.println(   "Zu indizierende Dateien im Verzeichnis: " + args[0] );
         System.out.println(   "Character-Encoding:                     " + args[1] );
         System.out.println(   "Indizierte Dateien:" );
         System.out.println( LuceneSimpleUtil.indexFiles( args[0], args[1], args[2], true ) );
      }
      if( args.length == 2 ) {
         System.out.println( "\nIndex im Verzeichnis: " + args[0] );
         System.out.println( "Suchwort '" + args[1] + "' gefunden in:" );
         List<Document> docs = LuceneSimpleUtil.searchFiles( args[0], args[1], 100 );
         for( int i = 0; i < docs.size(); i++ ) {
            System.out.println( docs.get( i ).get( "path" ) );
         }
      }
   }

   public static List<Document> searchFiles( String indexPath, String queryString, int maxResults ) throws Exception
   {
      if( queryString == null || queryString.trim().length() == 0 ) { return null; }
      List<Document> docs     = new ArrayList<Document>();
      IndexReader    reader   = DirectoryReader.open( FSDirectory.open( new File( indexPath ) ) );
      IndexSearcher  searcher = new IndexSearcher( reader );
      Analyzer       analyzer = new StandardAnalyzer( Version.LUCENE_4_9 );
      QueryParser    parser   = new QueryParser( Version.LUCENE_4_9, "contents", analyzer );
      Query          query    = parser.parse( queryString.trim() );
      TopDocs        results  = searcher.search( query, maxResults );
      ScoreDoc[]     hits     = results.scoreDocs;
      for( ScoreDoc scoreDoc : hits ) {
         docs.add( searcher.doc( scoreDoc.doc ) );
      }
      reader.close();
      analyzer.close();
      return docs;
   }

   public static List<File> indexFiles( String docsPath, String charEncoding, String indexPath, boolean createNew ) throws IOException
   {
      if( docsPath == null ) { return null; }
      final File docDir = new File( docsPath );
      if( !docDir.exists() || !docDir.canRead() ) { return null; }
      Directory dir = FSDirectory.open( new File( indexPath ) );
      Analyzer analyzer = new StandardAnalyzer( Version.LUCENE_4_9 );
      IndexWriterConfig iwc = new IndexWriterConfig( Version.LUCENE_4_9, analyzer );
      iwc.setOpenMode( ( createNew ) ? OpenMode.CREATE : OpenMode.CREATE_OR_APPEND );
      IndexWriter writer = new IndexWriter( dir, iwc );
      List<File> filesIndexed = indexDocs( writer, docDir, Charset.forName( charEncoding ) );
      writer.close();
      analyzer.close();
      dir.close();
      return filesIndexed;
   }

   private static List<File> indexDocs( IndexWriter writer, File file, Charset charEnc ) throws IOException
   {
      List<File> filesIndexed = new ArrayList<File>();
      if( file.canRead() ) {
         if( file.isDirectory() ) {
            String[] files = file.list();
            if( files != null ) {
               for( int i = 0; i < files.length; i++ ) {
                  filesIndexed.addAll( indexDocs( writer, new File( file, files[i] ), charEnc ) );
               }
            }
         } else {
            BufferedReader fileReader;
            try {
               fileReader = new BufferedReader( new InputStreamReader( new FileInputStream( file ), charEnc ) );
            } catch( FileNotFoundException fnfe ) {
               return filesIndexed;
            }
            try {
               Document doc = new Document();
               Field pathField = new StringField( "path", file.getPath(), Field.Store.YES );
               doc.add( pathField );
               doc.add( new LongField( "modified", file.lastModified(), Field.Store.NO ) );
               doc.add( new TextField( "contents", fileReader ) );
               if( writer.getConfig().getOpenMode() == OpenMode.CREATE ) {
                  writer.addDocument( doc );
               } else {
                  writer.updateDocument( new Term( "path", file.getPath() ), doc );
               }
               filesIndexed.add( file );
            } finally {
               fileReader.close();
            }
         }
      }
      return filesIndexed;
   }
}
